@@ -44,6 +44,7 @@ const TextEditor = require('./text-editor')
 const TextBuffer = require('text-buffer')
 const TextEditorRegistry = require('./text-editor-registry')
 const AutoUpdateManager = require('./auto-update-manager')
+const StartupTime = require('./startup-time')
 
 const stat = util.promisify(fs.stat)
 
@@ -306,7 +307,14 @@ class AtomEnvironment {
   }
 
   registerDefaultCommands () {
-    registerDefaultCommands({commandRegistry: this.commands, config: this.config, commandInstaller: this.commandInstaller, notificationManager: this.notifications, project: this.project, clipboard: this.clipboard})
+    registerDefaultCommands({
+      commandRegistry: this.commands,
+      config: this.config,
+      commandInstaller: this.commandInstaller,
+      notificationManager: this.notifications,
+      project: this.project,
+      clipboard: this.clipboard
+    })
   }
 
   registerDefaultOpeners () {
@@ -522,6 +530,18 @@ class AtomEnvironment {
     return this.loadTime
   }
 
+  // Public: Get the all the markers with the information about startup time.
+  //
+  // Returns an array of timing markers.
+  // Each timing is an object with two keys:
+  //  * `label`: string
+  //  * `time`:  Time since the `startTime` (in milliseconds).
+  getStartupMarkers () {
+    const data = StartupTime.exportData()
+
+    return data ? data.markers : []
+  }
+
   // Public: Get the load settings for the current window.
   //
   // Returns an {Object} containing all the load setting key/value pairs.
@@ -774,6 +794,8 @@ class AtomEnvironment {
 
   // Call this method when establishing a real application window.
   async startEditorWindow () {
+    StartupTime.addMarker('window:environment:start-editor-window:start')
+
     if (this.getLoadSettings().clearWindowState) {
       await this.stateStore.clear()
     }
@@ -784,7 +806,10 @@ class AtomEnvironment {
 
     const loadStatePromise = this.loadState().then(async state => {
       this.windowDimensions = state && state.windowDimensions
-      await this.displayWindow()
+      if (!this.getLoadSettings().headless) {
+        StartupTime.addMarker('window:environment:start-editor-window:display-window')
+        await this.displayWindow()
+      }
       this.commandInstaller.installAtomCommand(false, (error) => {
         if (error) console.warn(error.message)
       })
@@ -809,9 +834,11 @@ class AtomEnvironment {
 
       this.registerDefaultTargetForKeymaps()
 
+      StartupTime.addMarker('window:environment:start-editor-window:load-packages')
       this.packages.loadPackages()
 
       const startTime = Date.now()
+      StartupTime.addMarker('window:environment:start-editor-window:deserialize-state')
       await this.deserialize(state)
       this.deserializeTimings.atom = Date.now() - startTime
 
@@ -838,7 +865,7 @@ class AtomEnvironment {
           }
         }
         previousProjectPaths = newPaths
-        this.applicationDelegate.setRepresentedDirectoryPaths(newPaths)
+        this.applicationDelegate.setProjectRoots(newPaths)
       }))
       this.disposables.add(this.workspace.onDidDestroyPaneItem(({item}) => {
         const path = item.getPath && item.getPath()
@@ -847,12 +874,14 @@ class AtomEnvironment {
         }
       }))
 
+      StartupTime.addMarker('window:environment:start-editor-window:activate-packages')
       this.packages.activate()
       this.keymaps.loadUserKeymap()
       if (!this.getLoadSettings().safeMode) this.requireUserInitScript()
 
       this.menu.update()
 
+      StartupTime.addMarker('window:environment:start-editor-window:open-editor')
       await this.openInitialEmptyEditorIfNecessary()
     })
 
@@ -867,7 +896,11 @@ class AtomEnvironment {
       this.reopenProjectMenuManager.update()
     })
 
-    return Promise.all([loadStatePromise, loadHistoryPromise, updateProcessEnvPromise])
+    const output = await Promise.all([loadStatePromise, loadHistoryPromise, updateProcessEnvPromise])
+
+    StartupTime.addMarker('window:environment:start-editor-window:end')
+
+    return output
   }
 
   serialize (options) {
@@ -916,9 +949,9 @@ class AtomEnvironment {
 
   openInitialEmptyEditorIfNecessary () {
     if (!this.config.get('core.openEmptyEditorOnStart')) return
-    const {initialPaths} = this.getLoadSettings()
-    if (initialPaths && initialPaths.length === 0 && this.workspace.getPaneItems().length === 0) {
-      return this.workspace.open(null)
+    const {hasOpenFiles} = this.getLoadSettings()
+    if (!hasOpenFiles && this.workspace.getPaneItems().length === 0) {
+      return this.workspace.open(null, {pending: true})
     }
   }
 
@@ -1213,7 +1246,7 @@ or use Pane::saveItemAs for programmatic saving.`)
 
   loadState (stateKey) {
     if (this.enablePersistence) {
-      if (!stateKey) stateKey = this.getStateKey(this.getLoadSettings().initialPaths)
+      if (!stateKey) stateKey = this.getStateKey(this.getLoadSettings().initialProjectRoots)
       if (stateKey) {
         return this.stateStore.load(stateKey)
       } else {
@@ -1238,9 +1271,8 @@ or use Pane::saveItemAs for programmatic saving.`)
       try {
         await this.project.deserialize(state.project, this.deserializers)
       } catch (error) {
-        if (error.missingProjectPaths) {
-          missingProjectPaths.push(...error.missingProjectPaths)
-        } else {
+        // We handle the missingProjectPaths case in openLocations().
+        if (!error.missingProjectPaths) {
           this.notifications.addError('Unable to deserialize project', {
             description: error.message,
             stack: error.stack
@@ -1259,7 +1291,7 @@ or use Pane::saveItemAs for programmatic saving.`)
 
     if (missingProjectPaths.length > 0) {
       const count = missingProjectPaths.length === 1 ? '' : missingProjectPaths.length + ' '
-      const noun = missingProjectPaths.length === 1 ? 'directory' : 'directories'
+      const noun = missingProjectPaths.length === 1 ? 'folder' : 'folders'
       const toBe = missingProjectPaths.length === 1 ? 'is' : 'are'
       const escaped = missingProjectPaths.map(projectPath => `\`${projectPath}\``)
       let group
@@ -1388,7 +1420,7 @@ or use Pane::saveItemAs for programmatic saving.`)
           // Directory: add as a project folder
           foldersToAddToProject.add(this.project.getDirectoryForProjectPath(pathToOpen).getPath())
         } else if (stats.isFile()) {
-          if (location.mustBeDirectory) {
+          if (location.isDirectory) {
             // File: no longer a directory
             missingFolders.push(location)
           } else {
@@ -1403,7 +1435,7 @@ or use Pane::saveItemAs for programmatic saving.`)
         if (directory) {
           // Found: add as a project folder
           foldersToAddToProject.add(directory.getPath())
-        } else if (location.mustBeDirectory) {
+        } else if (location.isDirectory) {
           // Not found and must be a directory: add to missing list and use to derive state key
           missingFolders.push(location)
         } else {
